﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Input;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            StateChanged                          += Window_StateChanged;
            this.headerBorder.MouseLeftButtonDown += headerBorder_MouseLeftButtonDown;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 상태 변경시 처리하기 - Window_StateChanged(sender, e)

        /// <summary>
        /// 윈도우 상태 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_StateChanged(object sender, EventArgs e)
        {
            switch(WindowState)
            {
                case WindowState.Maximized : SetMaximizedState();         break;
                case WindowState.Normal    :
                case WindowState.Minimized : SetNormalOrMinimizedState(); break;
            }
        }

        #endregion
        #region 헤더 테두리 마우스 이동시 처리하기 - headerBorder_MouseMove(sender, e)

        /// <summary>
        /// 헤더 테두리 마우스 이동시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void headerBorder_MouseMove(object sender, MouseEventArgs e)
        {
            if(this.WindowState != WindowState.Maximized)
            {
                return;
            }

            if(Mouse.LeftButton != MouseButtonState.Pressed)
            {
                return;
            }

            this.WindowState = WindowState.Normal;

            this.Left = GetWindowLeft();
            this.Top  = 1;

            this.DragMove();
        }

        #endregion
        #region 헤더 테두리 마우스 왼쪽 버튼 DOWN 처리하기 - headerBorder_MouseLeftButtonDown(sender, e)

        /// <summary>
        /// 헤더 테두리 마우스 왼쪽 버튼 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void headerBorder_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if(e.ClickCount == 2)
            {
                SwitchState();
            }

            if((this.ResizeMode == ResizeMode.CanResize) || (this.ResizeMode == ResizeMode.CanResizeWithGrip))
            {
                try
                {
                    this.DragMove();
                }
                catch(Exception exception)
                {
                    Debug.WriteLine(exception);
                }
            }
        }

        #endregion
        #region 최소화 버튼 클릭시 처리하기 - minimizedButton_Click(sender, e)

        /// <summary>
        /// 최소화 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void minimizedButton_Click(object sender, RoutedEventArgs e)
        {
            this.WindowState = WindowState.Minimized;
        }

        #endregion
        #region 최대화 버튼 클릭시 처리하기 - maximizedButton_Click(sender, e)

        /// <summary>
        /// 최대화 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void maximizedButton_Click(object sender, RoutedEventArgs e)
        {
            if(WindowState == WindowState.Maximized)
            {
                WindowState = WindowState.Normal;
            }
            else
            {
                WindowState = WindowState.Maximized;
            }
        }

        #endregion
        #region 닫기 버튼 클릭시 처리하기 - closeButton_Click(sender, e)

        /// <summary>
        /// 닫기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void closeButton_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 최대화 상태 설정하기 - SetMaximizedState()

        /// <summary>
        /// 최대화 상태 설정하기
        /// </summary>
        private void SetMaximizedState()
        {
            this.headerBorder.MouseMove += headerBorder_MouseMove;
        }

        #endregion
        #region 일반 또는 최소화 상태 설정하기 - SetNormalOrMinimizedState()

        /// <summary>
        /// 일반 또는 최소화 상태 설정하기
        /// </summary>
        private void SetNormalOrMinimizedState()
        {
            this.headerBorder.MouseMove -= headerBorder_MouseMove;
        }

        #endregion
        #region 윈도우 왼쪽 구하기 - GetWindowLeft()

        /// <summary>
        /// 윈도우 왼쪽 구하기
        /// </summary>
        /// <returns>윈도우 왼쪽</returns>
        private double GetWindowLeft()
        {
            ScreenManager screenManager = ScreenManager.GetScreenManager(this);

            double leftBound  = screenManager.DesktopArea.Left;
            double rightBound = screenManager.DesktopArea.Right;

            Point currentPoint = GetMousePosition();

            double left = currentPoint.X - (this.Width / 2);

            if(left < leftBound)
            {
                return leftBound;
            }

            double right = currentPoint.X + (this.Width / 2);

            if(right > rightBound)
            {
                return rightBound - this.Width;
            }

            return left;
        }

        #endregion
        #region 마우스 위치 구하기 - GetMousePosition()

        /// <summary>
        /// 마우스 위치 구하기
        /// </summary>
        /// <returns>마우스 위치</returns>
        private Point GetMousePosition()
        {
            System.Drawing.Point point = System.Windows.Forms.Control.MousePosition;

            return new Point(point.X, point.Y);
        }

        #endregion
        #region 상태 전화하기 - SwitchState()

        /// <summary>
        /// 상태 전화하기
        /// </summary>
        private void SwitchState()
        {
            switch(WindowState)
            {
                case WindowState.Normal    : WindowState = WindowState.Maximized; break;
                case WindowState.Maximized : WindowState = WindowState.Normal;    break;
            }
        }

        #endregion
    }
}